<?php

namespace FiloBlu\ExtMagentoWidget\Model\ResourceModel\Layout;

/**
 * Layout update resource model
 */
class Update extends \Magento\Widget\Model\ResourceModel\Layout\Update
{

    protected $_customerSession;
    protected $_currentCustomerGroupId = -1;

    /**
     * @param \Magento\Framework\Model\ResourceModel\Db\Context $context
     * @param \Magento\Framework\Cache\FrontendInterface $cache
     * @param string $connectionName
     */
    public function __construct(
        \Magento\Framework\Model\ResourceModel\Db\Context $context,
        \Magento\Framework\Cache\FrontendInterface $cache,            
        \Magento\Customer\Model\Session $customerSession,
        $connectionName = null
           
    ) {
        $this->_customerSession = $customerSession;
        
        if($this->_customerSession->isLoggedIn()) {
            $this->_currentCustomerGroupId = $this->_customerSession->getCustomer()->getGroupId();
        }
        
        parent::__construct($context, $cache,$connectionName);
        
    }

    public function fetchUpdatesByHandle(
        $handle,
        \Magento\Framework\View\Design\ThemeInterface $theme,
        \Magento\Framework\App\ScopeInterface $store
    ) {
        $bind = ['theme_id' => $theme->getId(), 'store_id' => $store->getId(), 'customer_group_id' => $this->_currentCustomerGroupId];
        $cacheKey = implode('-', $bind);
        if (!isset($this->layoutUpdateCache[$cacheKey])) {
            $this->layoutUpdateCache[$cacheKey] = [];
            foreach ($this->getConnection()->fetchAll($this->_getFetchUpdatesByHandleSelect(), $bind) as $layout) {
                if (!isset($this->layoutUpdateCache[$cacheKey][$layout['handle']])) {
                    $this->layoutUpdateCache[$cacheKey][$layout['handle']] = '';
                }
                $this->layoutUpdateCache[$cacheKey][$layout['handle']] .= $layout['xml'];
            }
        }
        return isset($this->layoutUpdateCache[$cacheKey][$handle]) ? $this->layoutUpdateCache[$cacheKey][$handle] : '';
    }
    
    /**
     * Get select to fetch updates by handle
     *
     * @param bool $loadAllUpdates
     * @return \Magento\Framework\DB\Select
     */
    protected function _getFetchUpdatesByHandleSelect($loadAllUpdates = false)
    {
        //@todo Why it also loads layout updates for store_id=0, isn't it Admin Store View?
        //If 0 means 'all stores' why it then refers by foreign key to Admin in `store` and not to something named
        // 'All Stores'?

        $select = $this->getConnection()->select()->from(
            ['layout_update' => $this->getMainTable()],
            ['xml', 'handle']
        )->join(
            ['link' => $this->getTable('layout_link')],
            'link.layout_update_id=layout_update.layout_update_id',
            ''
        )->join(
            ['widget_instance_page_layout' => $this->getTable('widget_instance_page_layout')],
            'widget_instance_page_layout.layout_update_id=layout_update.layout_update_id',
            ''
        )->join(
            ['widget_instance_page' => $this->getTable('widget_instance_page')],
            'widget_instance_page.page_id=widget_instance_page_layout.page_id',
            ''
        )->join(
            ['widget_instance' => $this->getTable('widget_instance')],
            'widget_instance.instance_id=widget_instance_page.instance_id',
            ''
        )->where(
            'link.store_id IN (0, :store_id)'
        )->where(
            'link.theme_id = :theme_id'
        )->order(
            'layout_update.sort_order ' . \Magento\Framework\DB\Select::SQL_ASC
        );

        if (!$loadAllUpdates) {
            $select->where('link.is_temporary = 0');
        }
        
        $select->where('(widget_instance.customer_group IS NULL OR widget_instance.customer_group IN (:customer_group_id))');

        return $select;
    }

}
